# ================================================================

use strict;

# ================================================================

our($iLibraryVersion)='20211225';

# ================================================================

sub decodeAsURL
{
	my($aTarget)=@_;

	my($tResult)=$aTarget;

	$tResult=~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack('C',hex($1))/eg;
	$tResult=~ s/\+/ /g;

	return $tResult;
}

# ----------------------------------------------------------------

sub downloadFile
{
	my($aTargetURL,$aReferer,$aDirectory,$aUserAgent)=@_;

	use LWP::UserAgent;

	my($tBrowser)=LWP::UserAgent->new();
	$tBrowser->agent($aUserAgent);

	use HTTP::Request;

	my($tRequest1)=HTTP::Request->new('GET',$aTargetURL);
	$tRequest1->referer($aReferer);
	my($tResponse1)=$tBrowser->request($tRequest1);

	my($tResponseURL)=$tResponse1->request()->uri()->canonical();
	my($tContent)=$tResponse1->content();
	my($tFileName)=&getFileNameFromPathUnix($tResponseURL);

	if($tFileName ne '')
	{
		$tFileName=$tResponse1->filename();
	}

	my($tTargetFile)=$aDirectory.$tFileName;

	open(FILE,'>'.$tTargetFile);
	flock(FILE,2);

	print FILE $tContent;

	close(FILE);

	return $tTargetFile;
}

# ----------------------------------------------------------------

sub getCurrentTimeString
{
	my($tSecond,$tMinute,$tHour,$tDay,$tMonth,$tYear,$tWeekDay)=localtime(time);

	$tYear+=1900;
	$tMonth++;

	my(@tWeek)=('Sun','Mon','Tue','Wed','Thu','Fri','Sat');

	return sprintf('%04d/%02d/%02d %s %02d:%02d:%02d',$tYear,$tMonth,$tDay,$tWeek[$tWeekDay],$tHour,$tMinute,$tSecond);
}

# ----------------------------------------------------------------

sub getDirectoryFromPathUnix
{
	my($aTarget)=@_;

	return substr($aTarget,0,rindex($aTarget,'/')+1);
}

# ----------------------------------------------------------------

sub getDomainFromURL
{
	my($aURL)=@_;

	my($tResult)='';

	$aURL=~ s/http(s|):\/\/(.+?)\///g and $tResult=$2;

	return $tResult;
}

# ----------------------------------------------------------------

sub getExtensionUnix
{
	my($aTarget)=@_;

	my($tFileName)=&getFileNameFromPathUnix($aTarget);

	my(@tArray)=split(/\./,$tFileName);

	if(($#tArray)<=0)
	{
		return '';
	}
	else
	{
		return &toLowerCase($tArray[$#tArray]);
	}
}

# ----------------------------------------------------------------

sub getFileNameFromPathUnix
{
	my($aTarget)=@_;

	my(@tPathArray)=split(/\//,$aTarget);

	if(&isSame(substr($aTarget,-1,1),"\/"))
	{
		return '';
	}
	else
	{
		return $tPathArray[$#tPathArray];
	}
}

# ----------------------------------------------------------------

sub getFileSize
{
	my($aTarget)=@_;

	return -s $aTarget;
}

# ----------------------------------------------------------------

sub getQueryEntries
{
	my($aString,$aDoDecode)=@_;

	my(@tQuery)=split(/\&/,$aString);
	my(%tReturn)=();

	for(my($tCount1)=0;$tCount1<=($#tQuery);$tCount1++)
	{
		my($tName,$tValue)=split(/=/,$tQuery[$tCount1]);
		chomp($tValue);

		if($aDoDecode)
		{
			$tReturn{$tName}=&decodeAsURL($tValue);
		}
		else
		{
			$tReturn{$tName}=$tValue;
		}
	}

	return %tReturn;
}

# ----------------------------------------------------------------

sub hasSame
{
	my($aTarget,$aTargetArray)=@_;

	for(my($tCount1)=0;$tCount1<=($#$aTargetArray);$tCount1++)
	{
		if(&isSame($$aTargetArray[$tCount1],$aTarget))
		{
			return 1;
		}
	}

	return 0;
}

# ----------------------------------------------------------------

sub hasStarted
{
	my($aTarget,$aTargetArray)=@_;

	for(my($tCount1)=0;$tCount1<=($#$aTargetArray);$tCount1++)
	{
		if(&isStarted($aTarget,$$aTargetArray[$tCount1]))
		{
			return 1;
		}
	}

	return 0;
}

# ----------------------------------------------------------------

sub isContained
{
	my($aString1,$aString2)=@_;

	if(index($aString1,$aString2)>=0)
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

# ----------------------------------------------------------------

sub isDifferent
{
	my($aString1,$aString2)=@_;

	return ($aString1 ne $aString2);
}

# ----------------------------------------------------------------

sub isSame
{
	my($aString1,$aString2)=@_;

	return ($aString1 eq $aString2);
}

# ----------------------------------------------------------------

sub isStarted
{
	my($aString1,$aString2)=@_;

	if(index($aString1,$aString2)==0)
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

# ----------------------------------------------------------------

sub makeDirectory
{
	my($aDirectory)=@_;

	my($tResult)=1;

	my(@tDirectoryHierarchy)=split(/\//,$aDirectory);
	my($tTargetDirectory)='';

	for(my($tCount1)=0;$tCount1<=($#tDirectoryHierarchy);$tCount1++)
	{
		if($tDirectoryHierarchy[$tCount1] ne '')
		{
			$tTargetDirectory=$tTargetDirectory.'/'.$tDirectoryHierarchy[$tCount1];

			if(!-d $tTargetDirectory)
			{
				mkdir($tTargetDirectory) or $tResult=0;
			}
		}
	}

	return $tResult;
}

# ----------------------------------------------------------------

sub removeQuery
{
	my($aTarget)=@_;

	my($tBaseURL,$tQuery)=split(/\?/,$aTarget);

	return $tBaseURL;
}

# ----------------------------------------------------------------

sub removeReference
{
	my($aTarget)=@_;

	my($tBaseURL,$tReference)=split(/\#/,$aTarget);

	return $tBaseURL;
}

# ----------------------------------------------------------------

sub removeScheme
{
	my($aTarget)=@_;

	my($tScheme,$tRemaining)=split(/:\/\//,$aTarget);

	return $tRemaining;
}

# ----------------------------------------------------------------

sub toLowerCase
{
	my($aTarget)=@_;

	return lc($aTarget);
}

# ================================================================

1;
